//============================================
//			Test   SL013
//PIC C source code
//PIC16F887(8MHz) + PICC 9.70
//============================================
#include <htc.h>
#include <pic16f887.h>
#include <string.h>

__CONFIG(DEBUGEN&LVPDIS&FCMEN&IESOEN&BOREN&UNPROTECT&DUNPROTECT&MCLREN&PWRTEN&WDTDIS&INTIO);

//============================================
//  PIN define
//============================================
#define RX_PIN TRISC7
#define TX_PIN TRISC6
#define Buzzer RB2

//============================================
//  Uart status words define
//============================================
#define UARTSTATUS_FREE 0
#define UARTSTATUS_TX 1
#define UARTSTATUS_TXSUCC 2
#define UARTSTATUS_RXSUCC 3
#define UARTSTATUS_RXERR 4

//============================================
//  Card type define
//============================================
#define CARDTYPE_S50 0
#define CARDTYPE_S70 1
#define CARDTYPE_ProX 2
#define CARDTYPE_Pro 3
#define CARDTYPE_UL 4
#define CARDTYPE_DES 5
#define CARDTYPE_TAG 6
#define CARDTYPE_ICODE 7

//============================================
//  global variable define
//============================================
bank1 unsigned int g_iTimer;
bank1 unsigned char g_cTimer_Buzzer;
unsigned char g_bOverTime;
bit g_bUartTxHead;
bit g_bUartRxHead;
const unsigned char *g_pUartTxDat;
unsigned char g_cUartTxCheckSum;
unsigned char g_cUartRxCnt;
unsigned char g_cUartStatus;
bank1 unsigned char g_cRxBuf[40];
bank1 unsigned char g_cUartTxCnt;
bank1 unsigned char g_cUartTxDataLen;
bank1 unsigned char g_cCardType;

//============================================
//  Command List, preamble + length + command 
//============================================
const unsigned char Cmd_TrunOnRf[]={0x03,0x01,0x01};
const unsigned char Cmd_TrunOffRf[]={0x03,0x01,0x00};
const unsigned char Cmd_GetCSN[]={0x02,0x10};
const unsigned char Cmd_WriteBlock1[]={
  0x1A,0x12,0x00,0x01,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
  0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,
  0xAA,0xBB,0xCC,0xDD,0xEE,0xFF};
const unsigned char Cmd_ReadBlock1[]={
  0x0A,0x11,0x00,0x01,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
const unsigned char ComIntiPurse2[]={
  0x0E,0x13,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x78,0x56,0x34,0x12};
const unsigned char ComReadPurse2[]={
  0x0A,0x14,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};                              
const unsigned char ComIncrPurse2[]={
  0x0E,0x15,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x02,0x00,0x00,0x00};
const unsigned char ComDecrPurse2[]={
  0x0E,0x16,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x01,0x00,0x00,0x00};
const unsigned char pursevalue[4]={0x79,0x56,0x34,0x12};

//============================================
//  procedure define
//============================================
void Init_Hardware(void);
void BuzzerOn(void);
void Start_Time(unsigned int ms);
void SendBuf_UART(const unsigned char *dat,unsigned char len);
void Test_SL013();


//============================================
//  initialize system hardware config
//  timer,uart,port
//============================================
void Init_Hardware(void)
{	
  OSCCON=0x71;		//8M
  while(HTS==0);		//ȴȶ
  ANSEL=0x00;
  ANSELH=0x00;  
  TRISB2=0;
  OPTION=0x07;
  TMR0=0xb1;
  T0IE=1;
  RX_PIN=1;
  TX_PIN=1;
  Buzzer=0;
  TXSTA=0x04;
  RCSTA=0x90;
  BAUDCTL=0x08;
  SPBRG=0x67;
  SPBRGH=0;
  RCIE=1;
  TXEN=1;
  PEIE=1;
  GIE=1;
}

//============================================
//  interrupt service
//  timer interrupt
//  uart rx interrupt
//============================================
void interrupt isr(void)
{
  unsigned char dat;
  unsigned char CheckSum;
  unsigned char counter;
  if(T0IF)
  {
    T0IF=0;
    TMR0=0xb1;
    if(g_iTimer!=0)
    {
      g_iTimer--;
      if(g_iTimer==0) g_bOverTime=1;
    }
    if(g_cTimer_Buzzer!=0)
    {
      g_cTimer_Buzzer--;
      if(g_cTimer_Buzzer==0) Buzzer=0;
    }	
  }
  if(RCIF)
  {
    dat=RCREG;
    if(g_bUartRxHead)
    {
      if(dat!=0) g_cUartRxCnt=0;
      g_bUartRxHead=0;
    }
    else
    {
      g_cRxBuf[g_cUartRxCnt]=dat;
      g_cUartRxCnt++;
      if(dat==0xaa) g_bUartRxHead=1;
      if(g_cUartRxCnt==g_cRxBuf[0]+1)
      {
        CheckSum=0;
        for(counter=0;counter<g_cUartRxCnt;counter++) CheckSum=CheckSum^g_cRxBuf[counter];
        if(CheckSum==0) g_cUartStatus=UARTSTATUS_RXSUCC;
        else g_cUartStatus=UARTSTATUS_RXERR;
      }
    }	
  }
}

//============================================
//  trun on buzzer about 100mS
//============================================
void BuzzerOn(void)
{
  Buzzer=1;
  g_cTimer_Buzzer=10;
}

//============================================
//  start timer
//============================================
void Start_Time(unsigned int ms)
{
  g_iTimer=ms;
  g_bOverTime=0;
}

//============================================
//  send buffer by uart
//============================================
void SendBuf_UART(const unsigned char *dat,unsigned char len)
{
  unsigned char tmp;
  g_cUartStatus=UARTSTATUS_TX;
  g_pUartTxDat=dat;
  g_cUartTxDataLen=len;
  g_cUartTxCnt=0;
  g_cUartTxCheckSum=0;
  g_bUartTxHead=0;
  while(TXIF==0);
  TXREG=0xaa;
  while(TXIF==0);	
  TXREG=0xbb;
  while(g_cUartTxDataLen!=g_cUartTxCnt)
  {
    tmp=g_pUartTxDat[g_cUartTxCnt];
    g_cUartTxCheckSum=g_cUartTxCheckSum^tmp;
    g_cUartTxCnt++;
    while(TXIF==0); 
    TXREG=tmp;
    if(tmp==0xaa)
    {
      while(TXIF==0); 
      TXREG=0;
    }
  }
  while(TXIF==0);
  TXREG=g_cUartTxCheckSum;
  g_cUartStatus=UARTSTATUS_TXSUCC;
}

#define RCVCMD_SL013 g_cRxBuf[1]
#define RCVSTA_SL013 g_cRxBuf[2]

void Test_SL013(void)
{
  g_cCardType=0xff;
  SendBuf_UART(Cmd_GetCSN,sizeof(Cmd_GetCSN));
  Start_Time(300);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x10)||(RCVSTA_SL013!=0)) return;
  if(g_cRxBuf[7]==0) g_cCardType=CARDTYPE_S50;
  else if(g_cRxBuf[7]==1) g_cCardType=CARDTYPE_S70;
  else if(g_cRxBuf[7]==2) g_cCardType=CARDTYPE_ProX;
  if((g_cCardType==CARDTYPE_S50)||(g_cCardType==CARDTYPE_S70))
  {
    SendBuf_UART(Cmd_WriteBlock1,sizeof(Cmd_WriteBlock1));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x12)||(RCVSTA_SL013!=0)) return;
    SendBuf_UART(Cmd_ReadBlock1,sizeof(Cmd_ReadBlock1));
    Start_Time(80);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x11)||(RCVSTA_SL013!=0)) return;
    if(memcmp(&g_cRxBuf[3],&Cmd_WriteBlock1[10],16)!=0) return;
    SendBuf_UART(ComIntiPurse2,sizeof(ComIntiPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x13)||(RCVSTA_SL013!=0)) return;
    SendBuf_UART(ComIncrPurse2,sizeof(ComIncrPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x15)||(RCVSTA_SL013!=0)) return;
    SendBuf_UART(ComDecrPurse2,sizeof(ComDecrPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x16)||(RCVSTA_SL013!=0)) return;
    SendBuf_UART(ComReadPurse2,sizeof(ComReadPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x14)||(RCVSTA_SL013!=0)) return;
    if(memcmp(&pursevalue[0],&g_cRxBuf[3],4)!=0) return;
    BuzzerOn();
    while(g_cTimer_Buzzer!=0);
  }
}

void main(void)
{
  Init_Hardware();
  g_iTimer=0;
  while(1) Test_SL013();
}

